# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from .mainScreen import theTime  # mainScreen

from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.config import (
    config,
    getConfigListEntry,
    ConfigText,
    ConfigSelection,
    ConfigSubsection,
    ConfigYesNo,
    ConfigPassword,
)
from Plugins.Plugin import PluginDescriptor
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.VirtualKeyBoard import VirtualKeyBoard
from enigma import eTimer
from socket import gethostbyname
from twisted.internet import reactor
import os
import sys
import base64
import codecs
import re
import requests  # Usa requests per gestire le richieste HTTP

if sys.version_info[0] < 3:
    # Python 2
    # from __future__ import unicode_literals
    basestring = basestring
else:
    # Python 3
    basestring = str

try:
    bytes = bytes
    unicode = str
    from urllib.request import urlopen
    from urllib.request import Request
except ImportError:
    from urllib2 import urlopen
    from urllib2 import Request

global version
global sessions
global StrTim
global dyndnsservice
# global MainScreen
global listWidths
global DomainsAndServices


sessions = []
version = "1.30"
StrTim = theTime()

listWidths = [100]  # larghezza delle colonne list in percentuale (la somma, ovviamente, e' 100)
# MainScreen = mainScreen("b55 dynamic DNS service", version, listWidths)  # , [800, 600]) crea la skin

config.plugins.noipdyndns = ConfigSubsection()
config.plugins.noipdyndns.enable = ConfigYesNo(default=False)
config.plugins.noipdyndns.interval = ConfigSelection(default="60", choices=[("1", _("1 min.")),
                                                                            ("2", _("2 min.")),
                                                                            ("5", _("5 min.")),
                                                                            ("10", _("10 min.")),
                                                                            ("15", _("15 min.")),
                                                                            ("30", _("30 min.")),
                                                                            ("60", _("1 h.")),
                                                                            ("120", _("2 h.")),
                                                                            ("240", _("4 h")),
                                                                            ("600", _("10 h.")),
                                                                            ("1440", _("24 h."))])
config.plugins.noipdyndns.hostname = ConfigText(default="", fixed_size=False)

cnf = "%s/conf/plugin.conf" % os.path.dirname(sys.modules[__name__].__file__)

with open(cnf, "r") as f:
    domains = f.readlines()
list = []
deflt = ""
DomainsAndServices = []


try:
    for i in domains:
        data = i.split(",")
        if len(data) != 2:
            # print(StrTim.doLog("init - error loading conf file (" + cnf + ") - " + e.message, True))
            print('init - error loading conf file ')
        else:
            if deflt == "":
                deflt = data[0].strip()
        tmp = (data[0].strip(), _(data[0].strip()))
        list.append(tmp)
        DomainsAndServices.append(data)
except Exception as e:
    print("init - error loading conf file ", e)


config.plugins.noipdyndns.domain = ConfigSelection(default=deflt, choices=list)
config.plugins.noipdyndns.user = ConfigText(default="", fixed_size=False)
config.plugins.noipdyndns.password = ConfigPassword(default="", fixed_size=False)
# config.plugins.noipdyndns.logOn = ConfigYesNo(default=False)


class noipdyndnsScreenMain(ConfigListScreen, Screen):
    # global StrTim
    # global MainScreen

    def __init__(self, session, args=0):
        global dyndnsservice

        self.session = session
        Screen.__init__(self, session)
        # self.skin = MainScreen.skin

        skin = "%s/screens/main.xml" % os.path.dirname(sys.modules[__name__].__file__)
        with codecs.open(skin, "r", encoding="utf-8") as f:
            self.skin = f.read()

        # print('self.skin:', self.skin)
        self.list = []
        self.list.append(getConfigListEntry(_("Activate dynamic DNS update daemon"), config.plugins.noipdyndns.enable))
        self.list.append(getConfigListEntry(_("Interval for auto-update"), config.plugins.noipdyndns.interval))
        self.list.append(getConfigListEntry(_("Dynamic DNS Hostname"), config.plugins.noipdyndns.hostname))
        self.list.append(getConfigListEntry(_("Dynamic DNS Domain"), config.plugins.noipdyndns.domain))
        self.list.append(getConfigListEntry(_("DNS Admin Username"), config.plugins.noipdyndns.user))
        self.list.append(getConfigListEntry(_("DNS Admin Password"), config.plugins.noipdyndns.password))
        # self.list.append(getConfigListEntry(_("Activate events logging"), config.plugins.noipdyndns.logOn))
        ConfigListScreen.__init__(self, self.list)
        self["buttonred"] = Button(_("Cancel"))
        self["buttongreen"] = Button(_("Ok"))
        self["buttonyellow"] = Button(_("Set now"))
        self["buttonblue"] = Button(_("Info"))

        # dyndnsservice = os.popen('wget -qO - ifconfig.me').read().strip('\n\r')
        dyndnsservice.doIPverify()

        self["iplabel"] = Label(_("Your current outgoing IP: " + dyndnsservice.lastip))
        hn = str(config.plugins.noipdyndns.hostname.value) + "." + str(config.plugins.noipdyndns.domain.value)
        if len(hn) != 0:
            try:
                print("init - checking ip address for " + hn)
                self["namelabel"] = Label(_(hn + " IP address is " + gethostbyname(hn)))
                print("init - resolved to " + gethostbyname(hn))
            except Exception as e:
                self["namelabel"] = Label(_("Unable to Check IP address for " + hn))
                print("init - error checking ip address for " + hn + " - ", e)
        else:
            self["namelabel"] = Label("<no hostname>")

        self["setupActions"] = ActionMap(["SetupActions", "ColorActions", "VirtualKeyboardActions"],
                                         {"green": self.save,
                                          "red": self.cancel,
                                          "yellow": self.doIP,
                                          'showVirtualKeyboard': self.openKeyboard,
                                          "blue": self.doInfo,
                                          "save": self.save,
                                          "cancel": self.cancel,
                                          "ok": self.save}, -2)

    def openKeyboard(self):
        if self['config'].getCurrent()[1] == config.plugins.noipdyndns.hostname:
            self.session.openWithCallback(self.keybtxt, VirtualKeyBoard, title='Hostname', text='')
        if self['config'].getCurrent()[1] == config.plugins.noipdyndns.domain:
            self.session.openWithCallback(self.keybtxt, VirtualKeyBoard, title='Domain', text='')
        if self['config'].getCurrent()[1] == config.plugins.noipdyndns.user:
            self.session.openWithCallback(self.keybtxt, VirtualKeyBoard, title='Username', text='')
        if self['config'].getCurrent()[1] == config.plugins.noipdyndns.password:
            self.session.openWithCallback(self.keybtxt, VirtualKeyBoard, title='Password', text='')

    def keybtxt(self, callback=None):
        if callback is not None and len(callback):
            if self['config'].getCurrent()[1] == config.plugins.noipdyndns.hostname:
                config.plugins.noipdyndns.hostname.value = callback
            if self['config'].getCurrent()[1] == config.plugins.noipdyndns.domain:
                config.plugins.noipdyndns.domain.value = callback
            if self['config'].getCurrent()[1] == config.plugins.noipdyndns.user:
                config.plugins.noipdyndns.user.value = callback
            if self['config'].getCurrent()[1] == config.plugins.noipdyndns.password:
                config.plugins.noipdyndns.password.value = callback
        return

    def save(self):
        for x in self["config"].list:
            x[1].save()
        self.close(True)

    def cancel(self):
        for x in self["config"].list:
            x[1].cancel()
        self.close(False)

    def doIP(self):
        global dyndnsservice
        hn = str(config.plugins.noipdyndns.hostname.value) + "." + str(config.plugins.noipdyndns.domain.value)
        # dyndnsservice = os.popen('wget -qO - ifconfig.me').read().strip('\n\r')
        dyndnsservice.doIPverify
        risultato = str(dyndnsservice.doIPChange())
        if risultato.find("good") != -1:
            self.session.open(MessageBox, (_(hn + " succesfully changed to " + dyndnsservice.lastip)), MessageBox.TYPE_INFO, timeout=7)
        else:
            if risultato.find("noch") != -1:
                self.session.open(MessageBox, (_(hn + " already resolved to " + dyndnsservice.lastip)), MessageBox.TYPE_INFO, timeout=7)
            else:
                self.session.open(MessageBox, (_("Unable to change " + hn + " to " + dyndnsservice.lastip +
                                                 "\n" + risultato +
                                                 "\nverify account data!")), MessageBox.TYPE_INFO, timeout=7)

    def doInfo(self):
        self.session.open(MessageBox,
                          (_("Dynamic DNS service for no-ip & dyndns domains\n" +
                             "  by bartoma55 for SIFTeam E2 eXtreme Edition\n" +
                             "  many thanks to betatester mariorossi69\n" +
                             "  Modded from @lululla\n"
                             "\n" +
                             "  a) choose if service was enabled (daemon mode)\n" +
                             "  b) choose interval for daemon to check data\n" +
                             "  c) choose hostname to mask your IP\n" +
                             "  d) choose domain to mask your IP\n" +
                             "  e) choose username for DNS account\n" +
                             "  f) choose password for DNS account\n" +
                             "  red button = exit from plugin\n" +
                             "  green button = save configuration data\n" +
                             "  yellow button = try to set DNS data to server\n" +
                             "  blue button = show plugin info"
                             )), MessageBox.TYPE_INFO)


class noipdyndnsService:
    enabled = False
    sessions = []
    lastip = ""
    global DomainsAndServices

    def __init__(self):
        self.timer = eTimer()
        self.timer.callback.append(self.doTimer)
        if config.plugins.noipdyndns.enable.value:
            self.timer.start(int(config.plugins.noipdyndns.interval.value) * 60000)
        else:
            self.timer.stop()

    def enable(self):
        if config.plugins.noipdyndns.enable.value:
            self.enabled = True
            reactor.callLater(1, self.doIPverify)

    def disable(self):
        if self.enabled:
            self.timer.stop()
            self.enabled = False

    def addSession(self, session):
        self.sessions.append(session)

    def doTimer(self):
        self.timer.stop()
        self.doIPverify()

    def doURLquery(self, url):
        try:
            response = requests.get(url)
            response.raise_for_status()  # Verifica se ci sono errori HTTP
            return response.text
        except requests.RequestException as e:
            print("Error querying URL:", e)
            return ""

    def doIPverify(self):
        try:
            html = self.doURLquery("http://checkip.dyndns.org")
            # Usa regex per estrarre l'IP
            match = re.search(r'(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})', html)
            if match:
                strx = match.group(1)
            else:
                strx = "notabletogetip"
            if self.lastip != strx:
                self.lastip = strx
                print("IP found not equal to box IP, posting change IP request")
                reactor.callLater(1, self.doIPChange)
            else:
                print("IP found equal to box IP, no IP change request")

            if config.plugins.noipdyndns.enable.value:
                print("service enabled, starting timer")
                self.timer.start(int(config.plugins.noipdyndns.interval.value) * 60000)
            else:
                print("service not enabled, timer not started")

        except Exception as e:
            strx = "notabletogetip"
            if config.plugins.noipdyndns.enable.value:
                print("service enabled, starting timer ", e)
                self.timer.start(int(config.plugins.noipdyndns.interval.value) * 60000)
            else:
                print("service not enabled, timer not started")

    # def getDomainAndService(self, hostname):
        # # Dummy implementation, replace with your actual method
        # return hostname  # or "dyndns", etc.

    def getDomainAndService(self, hostnm):
        res = ""
        for i in DomainsAndServices:
            dm = i[0].strip()
            sv = i[1].strip()
            if (hostnm.find(dm) != -1):
                print("    - found - %s %s" % (dm, sv))
                res = sv
                break
        return res

    def doIPChange(self):
        try:
            hn = str(config.plugins.noipdyndns.hostname.value) + "." + str(config.plugins.noipdyndns.domain.value)
            srv = self.getDomainAndService(hn)
            if len(srv) == 0:
                return "Invalid domain or free DNS service, check conf file"

            if srv == "no-ip":
                url = "http://dynupdate.no-ip.com/nic/update?hostname=%s&myip=%s" % (hn, self.lastip)
            elif srv == "dyndns":
                url = "http://members.dyndns.org/nic/update?system=dyndns&hostname=%s&myip=%s&wildcard=ON&offline=NO" % (hn, self.lastip)
            else:
                return "Invalid DNS service (%s), check conf file" % (srv)

            risultato = self.doURLquery(url)
            if "good" in risultato:
                print("IP changed to DNS server")
            return risultato

        except Exception as e:
            print("IP was not changed in DNS server:", e)
            return "Unable to change IP to DNS server"

    # def doIPverify(self):
        # try:
            # # strx = os.popen('wget -qO - ifconfig.me').read().strip('\n\r')
            # html = self.doURLquery("http://checkip.dyndns.org")
            # strx = html.split("<body>")[1]
            # strx = strx.split("</body>")[0]
            # strx = strx.split(":")[1]
            # strx = strx.lstrip().rstrip()

            # if self.lastip != strx:
                # self.lastip = strx
                # print("IP found not equal to box IP, posting change IP request")
                # reactor.callLater(1, self.doIPChange)
            # else:
                # print("IP found equal to box IP, no IP change request")

            # if config.plugins.noipdyndns.enable.value:
                # print("service enabled, starting timer")
                # self.timer.start(int(config.plugins.noipdyndns.interval.value) * 60000)
            # else:
                # print("service not enabled, timer not started")

        # except Exception as e:
            # strx = "notabletogetip"
            # if config.plugins.noipdyndns.enable.value:
                # print("service enabled, starting timer ", e)
                # self.timer.start(int(config.plugins.noipdyndns.interval.value) * 60000)
            # else:
                # print("service not enabled, timer not started")

    # def getDomainAndService(self, hostnm):
        # res = ""
        # for i in DomainsAndServices:
            # dm = i[0].strip()
            # sv = i[1].strip()
            # if (hostnm.find(dm) != -1):
                # print("    - found - %s %s" % (dm, sv))
                # res = sv
                # break
        # return res

    # def doIPChange(self):
        # try:
            # hn = str(config.plugins.noipdyndns.hostname.value) + "." + str(config.plugins.noipdyndns.domain.value)
            # srv = self.getDomainAndService(hn)
            # if len(srv) == 0:
                # return "Invalid domain or free DNS service, check conf file"

            # if srv == "no-ip":
                # url = "http://dynupdate.no-ip.com/nic/update?hostname=%s&myip=%s" % (hn, self.lastip)
            # elif srv == "dyndns":
                # url = "http://members.dyndns.org/nic/update?system=dyndns&hostname=%s&myip=%s&wildcard=ON&offline=NO" % (hn, self.lastip)
            # else:
                # return "Invalid DNS service (%s), check conf file" % (srv)

            # risultato = self.doURLquery(url)
            # if risultato.find("good") != -1:
                # print("ip changed to DNS server")

            # return risultato

        # except Exception as e:
            # print("ip was not changed in DNS server ", e)
            # return "unable to change ip to DNS server"

    # def doURLquery(self, url):
        # print("  querying " + url)
        # try:
            # request = Request(url)
            # # base64string = encoded_data('%s:%s' % (config.plugins.noipdyndns.user.value, config.plugins.noipdyndns.password.value))[:-1]
            # if sys.version_info[0] < 3:  # Python 2
                # base64string = base64.encodestring(b"%s:%s" % (config.plugins.noipdyndns.user.value, config.plugins.noipdyndns.password.value))[:-1]
            # else:  # Python 3
                # base64string = base64.encodebytes(b"%s:%s" % (config.plugins.noipdyndns.user.value, config.plugins.noipdyndns.password.value))[:-1]
            # request.add_header("Authorization", "Basic %s" % base64string)
            # print("contacting DNS mantainer")
            # htmlFile = urlopen(request)
            # print("mantainer contacted")
            # htmlData = htmlFile.read()
            # print("request submitted")
            # htmlFile.close()
            # print("query terminated")
            # return htmlData

        # except Exception as e:
            # print("  error querying ", e)
            # strx = "notabletogetip"
            # return strx


dyndnsservice = noipdyndnsService()


def onPluginStart(session, **kwargs):
    session.openWithCallback(onPluginStartCB, noipdyndnsScreenMain)


def onPluginStartCB(changed):
    global dyndnsservice
    if changed:
        dyndnsservice.disable()
        dyndnsservice.enable()


def onSessionStart(reason, **kwargs):
    global dyndnsservice
    if config.plugins.noipdyndns.enable.value:
        if "session" in kwargs:
            dyndnsservice.addSession(kwargs["session"])
        if reason == 0:
            dyndnsservice.enable()
        elif reason == 1:
            dyndnsservice.disable()


def Plugins(path, **kwargs):
    return [PluginDescriptor(where=[PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc=onSessionStart),
            PluginDescriptor(name=_("noipdyndns"), description=_("Use <yourname>.<dynDNSdomain> for Your Router from Your Box"),
            where=[PluginDescriptor.WHERE_PLUGINMENU], fnc=onPluginStart, icon="plugin.png")]
